# -*- coding: utf8 -*-
from django.core.management.base import BaseCommand
from django.utils import timezone
from django.core.mail import EmailMessage
from django.conf import settings
from django.template.loader import render_to_string
from django.urls import reverse

from accounts.models import User

from datetime import timedelta
import sys


class Command(BaseCommand):
    help = "Cron pour la suppression des utilisateurs"

    def add_arguments(self, parser):
        parser.add_argument(
            "--dry-run",
            action="store_true",
            dest="dry_run",
            default=False,
            help="N'affiche que les actions qui devraient être effectuées",
        )

    def delete_user(self, user):
        if user.can_delete():
            try:
                user.delete()
            except Exception as e:
                self.return_code = 127
                if user.login_inp_net:
                    print("An error occurred when deleting user {}: {}".format(user.login_inp_net, e))
                else:
                    print("An error occurred when deleting user {}: {}".format(user.email, e))
        else:
            print("Unable to delete user {}".format(user.email))

    def send_mail(self, user):
        if not user.mail_delete:
            return
        url_profile = settings.BASE_URL + reverse("profile")
        content = render_to_string("emails/account_before_delete.txt", {"url_profile": url_profile})
        email = EmailMessage("Billetterie INP-net", content, settings.DEFAULT_FROM_EMAIL, [user.email])
        email.send()
        user.mail_delete = False
        user.delete_date = timezone.now() + timedelta(days=14)
        user.save()

    def handle(self, *args, **options):
        self.return_code = 0

        list_user_ldap_error = []
        list_user_ldap_delete = []
        list_user_delete = []
        list_user_warn = []
        interactif = sys.stdin.isatty() and sys.stdout.isatty()
        dry_run = options["dry_run"]

        for user in User.objects.all():
            if user.is_anonyme:
                continue
            elif user.login_inp_net:
                if user.get_LdapUser() is None:
                    if user.can_delete():
                        list_user_ldap_delete.append(user)
                    else:
                        list_user_ldap_error.append(user)
            else:  # Compte exte
                if user.delete_date < timezone.now():
                    if user.can_delete():
                        if user.mail_delete:
                            list_user_warn.append(user)
                        else:
                            list_user_delete.append(user)

        # affichage des comptes en erreur
        if list_user_ldap_error:
            print("Impossible de retrouver le compte ldap pour les logins suivants et impossible de les supprimer")
            for user in list_user_ldap_error:
                print(" - {}  ({})".format(user.login_inp_net, user.get_full_name()))

        # gestion des comptes ldap à supprimer
        if list_user_ldap_delete:
            print("Impossible de retrouver le compte ldap pour les logins suivants mais suppression possible")
            for user in list_user_ldap_delete:
                print(" - {}  ({})".format(user.login_inp_net, user.get_full_name()))
            if interactif and not dry_run:
                if input("Supprimer [o/N]") in ["o", "O"]:
                    for user in list_user_ldap_delete:
                        print("Suppression de {} ({})".format(user.login_inp_net, user.get_full_name()))
                        self.delete_user(user)
            else:
                print("Relancer ce script sans l'option dry-run pour les supprimer")

        # gestion des comptes exte à supprimer
        if list_user_delete:
            do_delete = not dry_run
            if interactif or dry_run:
                print("Compte Exté à supprimer")
                for user in list_user_delete:
                    print(" - {}  ({})".format(user.email, user.get_full_name()))
                if not dry_run:
                    if input("Supprimer [o/N]") not in ["o", "O"]:
                        do_delete = False
                else:
                    print("Relancer ce script sans l'option dry-run pour les supprimer")

            if do_delete:
                for user in list_user_delete:
                    print("Suppression de {} ({})".format(user.email, user.get_full_name()))
                    self.delete_user(user)

        # gestion des comptes mails à envoyer
        if list_user_warn:
            do_warn = not dry_run
            if interactif or dry_run:
                print("Envoie des mails d'avertissement de suppression aux utilisateurs suivants")
                for user in list_user_warn:
                    print(" - {}  ({})".format(user.email, user.get_full_name()))
                if not dry_run:
                    if input("Envoyer et programmer la suppression dans 14 jours [O/n]") not in ["", "o", "O"]:
                        do_warn = False
                else:
                    print("Relancer ce script sans l'option dry-run pour les envoyer les mails")

            if do_warn:
                for user in list_user_warn:
                    print("Envoie du mail d'avertissement de suppression à {}".format(user.email))
                    try:
                        self.send_mail(user)
                    except Exception as e:
                        self.return_code = 127
                        print("An error occurred while sending mail to user {}: {}".format(user.email, e))

        if self.return_code != 0:
            sys.exit(self.return_code)
